# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, List, Tuple, cast

import pytest

from giskard_hub import HubClient, AsyncHubClient
from tests.utils import assert_matches_type
from giskard_hub.types import (
    KnowledgeBase,
    APIPaginatedMetadata,
    KnowledgeBaseDocumentRow,
    KnowledgeBaseDocumentDetail,
    KnowledgeBaseDocumentFiltersParam,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestKnowledgeBases:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: HubClient) -> None:
        knowledge_base = client.knowledge_bases.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: HubClient) -> None:
        knowledge_base = client.knowledge_bases.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
            description="description",
            document_column="document_column",
            topic_column="topic_column",
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: HubClient) -> None:
        response = client.knowledge_bases.with_raw_response.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: HubClient) -> None:
        with client.knowledge_bases.with_streaming_response.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: HubClient) -> None:
        knowledge_base = client.knowledge_bases.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: HubClient) -> None:
        response = client.knowledge_bases.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: HubClient) -> None:
        with client.knowledge_bases.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `knowledge_base_id` but received ''",
        ):
            client.knowledge_bases.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: HubClient) -> None:
        knowledge_base = client.knowledge_bases.update(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: HubClient) -> None:
        knowledge_base = client.knowledge_bases.update(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            status={
                "total": 0,
                "current": 0,
                "error": "error",
                "state": "skipped",
            },
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: HubClient) -> None:
        response = client.knowledge_bases.with_raw_response.update(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: HubClient) -> None:
        with client.knowledge_bases.with_streaming_response.update(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `knowledge_base_id` but received ''",
        ):
            client.knowledge_bases.with_raw_response.update(
                knowledge_base_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: HubClient) -> None:
        knowledge_base = client.knowledge_bases.list()
        assert_matches_type(List[KnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: HubClient) -> None:
        knowledge_base = client.knowledge_bases.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[KnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: HubClient) -> None:
        response = client.knowledge_bases.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(List[KnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: HubClient) -> None:
        with client.knowledge_bases.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(List[KnowledgeBase], knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: HubClient) -> None:
        knowledge_base = client.knowledge_bases.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(type(None), knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: HubClient) -> None:
        response = client.knowledge_bases.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(type(None), knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: HubClient) -> None:
        with client.knowledge_bases.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(type(None), knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `knowledge_base_id` but received ''",
        ):
            client.knowledge_bases.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_delete(self, client: HubClient) -> None:
        knowledge_base = client.knowledge_bases.bulk_delete(
            knowledge_base_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(type(None), knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_bulk_delete(self, client: HubClient) -> None:
        response = client.knowledge_bases.with_raw_response.bulk_delete(
            knowledge_base_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(type(None), knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_bulk_delete(self, client: HubClient) -> None:
        with client.knowledge_bases.with_streaming_response.bulk_delete(
            knowledge_base_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(type(None), knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_documents(self, client: HubClient) -> None:
        knowledge_base: List[KnowledgeBaseDocumentRow] = client.knowledge_bases.search_documents(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[KnowledgeBaseDocumentRow], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_documents_with_all_params(self, client: HubClient) -> None:
        knowledge_base: List[KnowledgeBaseDocumentRow] = client.knowledge_bases.search_documents(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            query="query",
            filters=cast(KnowledgeBaseDocumentFiltersParam, {}),
            limit=20,
            offset=0,
        )
        assert_matches_type(List[KnowledgeBaseDocumentRow], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_documents_with_metadata(self, client: HubClient) -> None:
        knowledge_base = client.knowledge_bases.search_documents(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include_metadata=True,
        )
        assert_matches_type(
            Tuple[List[KnowledgeBaseDocumentRow], APIPaginatedMetadata], knowledge_base, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_search_documents(self, client: HubClient) -> None:
        response = client.knowledge_bases.with_raw_response.search_documents(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(List[KnowledgeBaseDocumentRow], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_search_documents(self, client: HubClient) -> None:
        with client.knowledge_bases.with_streaming_response.search_documents(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(List[KnowledgeBaseDocumentRow], knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_search_documents(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `knowledge_base_id` but received ''",
        ):
            client.knowledge_bases.with_raw_response.search_documents(
                knowledge_base_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_document(self, client: HubClient) -> None:
        knowledge_base = client.knowledge_bases.retrieve_document(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(KnowledgeBaseDocumentDetail, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_document(self, client: HubClient) -> None:
        response = client.knowledge_bases.with_raw_response.retrieve_document(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBaseDocumentDetail, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_document(self, client: HubClient) -> None:
        with client.knowledge_bases.with_streaming_response.retrieve_document(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBaseDocumentDetail, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve_document(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `knowledge_base_id` but received ''",
        ):
            client.knowledge_bases.with_raw_response.retrieve_document(
                knowledge_base_id="",
                document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `document_id` but received ''",
        ):
            client.knowledge_bases.with_raw_response.retrieve_document(
                knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                document_id="",
            )


class TestAsyncKnowledgeBases:
    parametrize = pytest.mark.parametrize(
        "async_client",
        [False, True, {"http_client": "aiohttp"}],
        indirect=True,
        ids=["loose", "strict", "aiohttp"],
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
            description="description",
            document_column="document_column",
            topic_column="topic_column",
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncHubClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncHubClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncHubClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncHubClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `knowledge_base_id` but received ''",
        ):
            await async_client.knowledge_bases.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.update(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.update(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            status={
                "total": 0,
                "current": 0,
                "error": "error",
                "state": "skipped",
            },
        )
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHubClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.update(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHubClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.update(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBase, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `knowledge_base_id` but received ''",
        ):
            await async_client.knowledge_bases.with_raw_response.update(
                knowledge_base_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.list()
        assert_matches_type(List[KnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[KnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHubClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(List[KnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHubClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(List[KnowledgeBase], knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(type(None), knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncHubClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(type(None), knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncHubClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(type(None), knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `knowledge_base_id` but received ''",
        ):
            await async_client.knowledge_bases.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_delete(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.bulk_delete(
            knowledge_base_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(type(None), knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_bulk_delete(self, async_client: AsyncHubClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.bulk_delete(
            knowledge_base_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(type(None), knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_delete(self, async_client: AsyncHubClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.bulk_delete(
            knowledge_base_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(type(None), knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_documents(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.search_documents(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[KnowledgeBaseDocumentRow], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_documents_with_all_params(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.search_documents(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filters={},
            limit=0,
            offset=0,
        )
        assert_matches_type(List[KnowledgeBaseDocumentRow], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_documents_with_metadata(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.search_documents(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include_metadata=True,
        )
        assert_matches_type(
            Tuple[List[KnowledgeBaseDocumentRow], APIPaginatedMetadata], knowledge_base, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_search_documents(self, async_client: AsyncHubClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.search_documents(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(List[KnowledgeBaseDocumentRow], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_search_documents(self, async_client: AsyncHubClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.search_documents(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(List[KnowledgeBaseDocumentRow], knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_search_documents(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `knowledge_base_id` but received ''",
        ):
            await async_client.knowledge_bases.with_raw_response.search_documents(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_document(self, async_client: AsyncHubClient) -> None:
        knowledge_base = await async_client.knowledge_bases.retrieve_document(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(KnowledgeBaseDocumentDetail, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_document(self, async_client: AsyncHubClient) -> None:
        response = await async_client.knowledge_bases.with_raw_response.retrieve_document(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBaseDocumentDetail, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_document(self, async_client: AsyncHubClient) -> None:
        async with async_client.knowledge_bases.with_streaming_response.retrieve_document(
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBaseDocumentDetail, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve_document(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `knowledge_base_id` but received ''",
        ):
            await async_client.knowledge_bases.with_raw_response.retrieve_document(
                knowledge_base_id="",
                document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `document_id` but received ''",
        ):
            await async_client.knowledge_bases.with_raw_response.retrieve_document(
                knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                document_id="",
            )



from __future__ import annotations

from typing import List, Literal, Iterable, Optional

import httpx

from ...types import BulkMoveTestCasesParams
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .comments import (
    CommentsResource,
    AsyncCommentsResource,
    CommentsResourceWithRawResponse,
    AsyncCommentsResourceWithRawResponse,
    CommentsResourceWithStreamingResponse,
    AsyncCommentsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.chat import ChatMessageParam, ChatMessageWithMetadataParam
from ...types.check import CheckConfigParam, _check_params_to_api
from ..._base_client import make_request_options
from ...types.common import APIResponse
from ...types.test_case import (
    TestCase,
    TestCaseCreateParams,
    TestCaseUpdateParams,
    TestCaseBulkDeleteParams,
    TestCaseBulkUpdateParams,
)

__all__ = ["TestCasesResource", "AsyncTestCasesResource"]

DemoOutput = ChatMessageWithMetadataParam | str


def _normalize_demo_output(
    demo_output: Optional[ChatMessageWithMetadataParam | str] | Omit,
) -> Optional[ChatMessageWithMetadataParam] | Omit:
    if isinstance(demo_output, (Omit, type(None))):
        return demo_output
    if isinstance(demo_output, str):
        return ChatMessageWithMetadataParam(role="assistant", content=demo_output)
    return demo_output


class TestCasesResource(SyncAPIResource):
    __test__ = False

    @cached_property
    def comments(self) -> CommentsResource:
        return CommentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> TestCasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return TestCasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TestCasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return TestCasesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        dataset_id: str,
        messages: Iterable[ChatMessageParam],
        checks: Iterable[CheckConfigParam] | Omit = omit,
        demo_output: Optional[DemoOutput] | Omit = omit,
        status: Optional[Literal["active", "draft"]] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCase:
        """
        Create a new test case in a dataset with conversation messages and optional checks.

        Parameters
        ----------
        dataset_id : str
            Dataset ID to create the test case from.
        messages : Iterable[ChatMessageParam]
            Messages to add to the test case.
        checks : Iterable[CheckConfigParam] | Omit
            Checks to add to the test case. Each check should have an ``identifier``
            and optionally ``params`` (check-specific fields) and ``enabled``.
        demo_output : Optional[DemoOutput] | Omit
            Agent output. Can be a plain string or a ``ChatMessageWithMetadataParam`` dict.
        status : Optional[Literal["active", "draft"]] | Omit
            Status of the test case.
        tags : SequenceNotStr[str] | Omit
            Tags to apply to the test case.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCase
            The newly created test case.
        """
        api_checks: Iterable[object] | Omit = _check_params_to_api(checks) if not isinstance(checks, Omit) else omit
        api_demo_output = _normalize_demo_output(demo_output)
        response = self._post(
            "/v2/test-cases",
            body=maybe_transform(
                {
                    "dataset_id": dataset_id,
                    "messages": messages,
                    "checks": api_checks,
                    "demo_output": api_demo_output,
                    "status": status,
                    "tags": tags,
                },
                TestCaseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCase],
        )

        return self._unwrap(response)

    def retrieve(
        self,
        test_case_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCase:
        """
        Retrieve a test case by its ID.

        Parameters
        ----------
        test_case_id : str
            Test Case ID to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCase
            The retrieved test case.

        Raises
        ------
        ValueError
            If ``test_case_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        response = self._get(
            f"/v2/test-cases/{test_case_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCase],
        )

        return self._unwrap(response)

    def update(
        self,
        test_case_id: str,
        *,
        checks: Optional[Iterable[CheckConfigParam]] | Omit = omit,
        dataset_id: Optional[str] | Omit = omit,
        demo_output: Optional[DemoOutput] | Omit = omit,
        messages: Optional[Iterable[ChatMessageParam]] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        status: Optional[Literal["active", "draft"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCase:
        """
        Update an existing test case's messages, checks, tags, or status.

        Parameters
        ----------
        test_case_id : str
            Test Case ID to update.
        checks : Optional[Iterable[CheckConfigParam]] | Omit
            Checks to update the test case. Each check should have an ``identifier``
            and optionally ``params`` (check-specific fields) and ``enabled``.
        dataset_id : Optional[str] | Omit
            Dataset ID to update the test case.
        demo_output : Optional[DemoOutput] | Omit
            Agent output. Can be a plain string or a ``ChatMessageWithMetadataParam`` dict.
        messages : Optional[Iterable[ChatMessageParam]] | Omit
            Messages to update the test case.
        tags : Optional[SequenceNotStr[str]] | Omit
            Tags to update the test case.
        status : Optional[Literal["active", "draft"]] | Omit
            Status to update of the test case.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCase
            The updated test case.

        Raises
        ------
        ValueError
            If ``test_case_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        api_checks: Iterable[object] | Omit | None
        if checks is None or isinstance(checks, Omit):
            api_checks = checks  # type: ignore[assignment]
        else:
            api_checks = _check_params_to_api(checks)
        api_demo_output = _normalize_demo_output(demo_output)
        response = self._patch(
            f"/v2/test-cases/{test_case_id}",
            body=maybe_transform(
                {
                    "checks": api_checks,
                    "dataset_id": dataset_id,
                    "demo_output": api_demo_output,
                    "messages": messages,
                    "tags": tags,
                    "status": status,
                },
                TestCaseUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCase],
        )

        return self._unwrap(response)

    def delete(
        self,
        test_case_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a test case by its ID.

        Parameters
        ----------
        test_case_id : str
            Test Case ID to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``test_case_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        response = self._delete(
            f"/v2/test-cases/{test_case_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def bulk_delete(
        self,
        *,
        test_case_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete multiple test cases at once.

        Parameters
        ----------
        test_case_ids : SequenceNotStr[str]
            Test Case IDs to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = self._delete(
            "/v2/test-cases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"test_case_ids": test_case_ids}, TestCaseBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def bulk_update(
        self,
        *,
        test_case_ids: SequenceNotStr[str],
        disabled_checks: Optional[SequenceNotStr[str]] | Omit = omit,
        enabled_checks: Optional[SequenceNotStr[str]] | Omit = omit,
        added_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        removed_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        status: Optional[Literal["active", "draft"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[TestCase]:
        """
        Bulk update multiple test cases' checks, tags, or status.

        Parameters
        ----------
        test_case_ids : SequenceNotStr[str]
            Test Case IDs to update.
        disabled_checks : Optional[SequenceNotStr[str]] | Omit
            Partial list of checks to be disabled.
        enabled_checks : Optional[SequenceNotStr[str]] | Omit
            Partial list of checks to be enabled.
        added_tags : Optional[SequenceNotStr[str]] | Omit
            Tags to be added to the test cases.
        removed_tags : Optional[SequenceNotStr[str]] | Omit
            Tags to be removed from the test cases.
        status : Optional[Literal["active", "draft"]] | Omit
            Status of the test cases.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[TestCase]
            The updated test cases.
        """
        response = self._patch(
            "/v2/test-cases",
            body=maybe_transform(
                {
                    "ids": test_case_ids,
                    "disabled_checks": disabled_checks,
                    "enabled_checks": enabled_checks,
                    "added_tags": added_tags,
                    "removed_tags": removed_tags,
                    "status": status,
                },
                TestCaseBulkUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[TestCase]],
        )

        return self._unwrap(response)

    def bulk_move(
        self,
        *,
        test_case_ids: List[str],
        target_dataset_id: str,
        duplicate: Optional[bool] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Move or copy test cases between datasets.

        Parameters
        ----------
        test_case_ids : List[str]
            List of test case IDs to move.
        target_dataset_id : str
            Target dataset ID to move test cases to.
        duplicate : Optional[bool] | Omit
            If true, keep a copy of the test cases in the original dataset. Default is true.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = self._post(
            "/v2/test-cases/bulk-move",
            body=maybe_transform(
                {
                    "chat_test_case_ids": test_case_ids,
                    "dataset_id": target_dataset_id,
                    "duplicate": duplicate,
                },
                BulkMoveTestCasesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)


class AsyncTestCasesResource(AsyncAPIResource):
    @cached_property
    def comments(self) -> AsyncCommentsResource:
        return AsyncCommentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTestCasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTestCasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTestCasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncTestCasesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        dataset_id: str,
        messages: Iterable[ChatMessageParam],
        checks: Iterable[CheckConfigParam] | Omit = omit,
        demo_output: Optional[DemoOutput] | Omit = omit,
        status: Optional[Literal["active", "draft"]] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCase:
        """
        Create a new test case in a dataset with conversation messages and optional checks.

        Parameters
        ----------
        dataset_id : str
            Dataset ID to create the test case for.
        messages : Iterable[ChatMessageParam]
            Messages to add to the test case.
        checks : Iterable[CheckConfigParam] | Omit
            Checks to add to the test case. Each check should have an ``identifier``
            and optionally ``params`` (check-specific fields) and ``enabled``.
        demo_output : Optional[DemoOutput] | Omit
            Agent output. Can be a plain string or a ``ChatMessageWithMetadataParam`` dict.
        status : Optional[Literal["active", "draft"]] | Omit
            Status of the test case.
        tags : SequenceNotStr[str] | Omit
            Tags to apply to the test case.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCase
            The newly created test case.
        """
        api_checks: Iterable[object] | Omit = _check_params_to_api(checks) if not isinstance(checks, Omit) else omit
        api_demo_output = _normalize_demo_output(demo_output)
        response = await self._post(
            "/v2/test-cases",
            body=await async_maybe_transform(
                {
                    "dataset_id": dataset_id,
                    "messages": messages,
                    "checks": api_checks,
                    "demo_output": api_demo_output,
                    "status": status,
                    "tags": tags,
                },
                TestCaseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCase],
        )

        return self._unwrap(response)

    async def retrieve(
        self,
        test_case_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCase:
        """
        Retrieve a test case by its ID.

        Parameters
        ----------
        test_case_id : str
            Test Case ID to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCase
            The retrieved test case.

        Raises
        ------
        ValueError
            If ``test_case_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        response = await self._get(
            f"/v2/test-cases/{test_case_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCase],
        )

        return self._unwrap(response)

    async def update(
        self,
        test_case_id: str,
        *,
        checks: Optional[Iterable[CheckConfigParam]] | Omit = omit,
        dataset_id: Optional[str] | Omit = omit,
        demo_output: Optional[DemoOutput] | Omit = omit,
        messages: Optional[Iterable[ChatMessageParam]] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        status: Optional[Literal["active", "draft"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCase:
        """
        Update an existing test case's messages, checks, tags, or status.

        Parameters
        ----------
        test_case_id : str
            Test Case ID to update.
        checks : Optional[Iterable[CheckConfigParam]] | Omit
            Checks to update the test case. Each check should have an ``identifier``
            and optionally ``params`` (check-specific fields) and ``enabled``.
        dataset_id : Optional[str] | Omit
            Dataset ID to update the test case.
        demo_output : Optional[DemoOutput] | Omit
            Agent output. Can be a plain string or a ``ChatMessageWithMetadataParam`` dict.
        messages : Optional[Iterable[ChatMessageParam]] | Omit
            Messages to update the test case.
        tags : Optional[SequenceNotStr[str]] | Omit
            Tags to update the test case.
        status : Optional[Literal["active", "draft"]] | Omit
            Status to update of the test case.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCase
            The updated test case.

        Raises
        ------
        ValueError
            If ``test_case_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        api_checks: Iterable[object] | Omit | None
        if checks is None or isinstance(checks, Omit):
            api_checks = checks  # type: ignore[assignment]
        else:
            api_checks = _check_params_to_api(checks)
        api_demo_output = _normalize_demo_output(demo_output)
        response = await self._patch(
            f"/v2/test-cases/{test_case_id}",
            body=await async_maybe_transform(
                {
                    "checks": api_checks,
                    "dataset_id": dataset_id,
                    "demo_output": api_demo_output,
                    "messages": messages,
                    "tags": tags,
                    "status": status,
                },
                TestCaseUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCase],
        )

        return self._unwrap(response)

    async def delete(
        self,
        test_case_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a test case by its ID.

        Parameters
        ----------
        test_case_id : str
            Test Case ID to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``test_case_id`` is empty.
        """
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        response = await self._delete(
            f"/v2/test-cases/{test_case_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def bulk_delete(
        self,
        *,
        test_case_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete multiple test cases at once.

        Parameters
        ----------
        test_case_ids : SequenceNotStr[str]
            Test Case IDs to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = await self._delete(
            "/v2/test-cases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"test_case_ids": test_case_ids}, TestCaseBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def bulk_update(
        self,
        *,
        ids: SequenceNotStr[str],
        disabled_checks: Optional[SequenceNotStr[str]] | Omit = omit,
        enabled_checks: Optional[SequenceNotStr[str]] | Omit = omit,
        added_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        removed_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        status: Optional[Literal["active", "draft"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[TestCase]:
        """
        Bulk update multiple test cases' checks, tags, or status.

        Parameters
        ----------
        test_case_ids : SequenceNotStr[str]
            Test Case IDs to update.
        disabled_checks : Optional[SequenceNotStr[str]] | Omit
            Partial list of checks to be disabled.
        enabled_checks : Optional[SequenceNotStr[str]] | Omit
            Partial list of checks to be enabled.
        added_tags : Optional[SequenceNotStr[str]] | Omit
            Tags to be added to the test cases.
        removed_tags : Optional[SequenceNotStr[str]] | Omit
            Tags to be removed from the test cases.
        status : Optional[Literal["active", "draft"]] | Omit
            Status of the test cases.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[TestCase]
            The updated test cases.
        """
        response = await self._patch(
            "/v2/test-cases",
            body=await async_maybe_transform(
                {
                    "ids": ids,
                    "disabled_checks": disabled_checks,
                    "enabled_checks": enabled_checks,
                    "added_tags": added_tags,
                    "removed_tags": removed_tags,
                    "status": status,
                },
                TestCaseBulkUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[TestCase]],
        )

        return self._unwrap(response)

    async def bulk_move(
        self,
        *,
        test_case_ids: List[str],
        target_dataset_id: str,
        duplicate: Optional[bool] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Move or copy test cases between datasets.

        Parameters
        ----------
        test_case_ids : List[str]
            List of test case IDs to move.
        target_dataset_id : str
            Target dataset ID to move test cases to.
        duplicate : Optional[bool] | Omit
            If true, keep a copy of the test cases in the original dataset. Default is true.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = await self._post(
            "/v2/test-cases/bulk-move",
            body=await async_maybe_transform(
                {
                    "chat_test_case_ids": test_case_ids,
                    "dataset_id": target_dataset_id,
                    "duplicate": duplicate,
                },
                BulkMoveTestCasesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)


class TestCasesResourceWithRawResponse:
    __test__ = False

    def __init__(self, test_cases: TestCasesResource) -> None:
        self._test_cases = test_cases

        self.create = to_raw_response_wrapper(
            test_cases.create,
        )
        self.retrieve = to_raw_response_wrapper(
            test_cases.retrieve,
        )
        self.update = to_raw_response_wrapper(
            test_cases.update,
        )
        self.delete = to_raw_response_wrapper(
            test_cases.delete,
        )
        self.bulk_delete = to_raw_response_wrapper(
            test_cases.bulk_delete,
        )
        self.bulk_update = to_raw_response_wrapper(
            test_cases.bulk_update,
        )
        self.bulk_move = to_raw_response_wrapper(
            test_cases.bulk_move,
        )

    @cached_property
    def comments(self) -> CommentsResourceWithRawResponse:
        return CommentsResourceWithRawResponse(self._test_cases.comments)


class AsyncTestCasesResourceWithRawResponse:
    def __init__(self, test_cases: AsyncTestCasesResource) -> None:
        self._test_cases = test_cases

        self.create = async_to_raw_response_wrapper(
            test_cases.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            test_cases.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            test_cases.update,
        )
        self.delete = async_to_raw_response_wrapper(
            test_cases.delete,
        )
        self.bulk_delete = async_to_raw_response_wrapper(
            test_cases.bulk_delete,
        )
        self.bulk_update = async_to_raw_response_wrapper(
            test_cases.bulk_update,
        )
        self.bulk_move = async_to_raw_response_wrapper(
            test_cases.bulk_move,
        )

    @cached_property
    def comments(self) -> AsyncCommentsResourceWithRawResponse:
        return AsyncCommentsResourceWithRawResponse(self._test_cases.comments)


class TestCasesResourceWithStreamingResponse:
    __test__ = False

    def __init__(self, test_cases: TestCasesResource) -> None:
        self._test_cases = test_cases

        self.create = to_streamed_response_wrapper(
            test_cases.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            test_cases.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            test_cases.update,
        )
        self.delete = to_streamed_response_wrapper(
            test_cases.delete,
        )
        self.bulk_delete = to_streamed_response_wrapper(
            test_cases.bulk_delete,
        )
        self.bulk_update = to_streamed_response_wrapper(
            test_cases.bulk_update,
        )
        self.bulk_move = to_streamed_response_wrapper(
            test_cases.bulk_move,
        )

    @cached_property
    def comments(self) -> CommentsResourceWithStreamingResponse:
        return CommentsResourceWithStreamingResponse(self._test_cases.comments)


class AsyncTestCasesResourceWithStreamingResponse:
    def __init__(self, test_cases: AsyncTestCasesResource) -> None:
        self._test_cases = test_cases

        self.create = async_to_streamed_response_wrapper(
            test_cases.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            test_cases.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            test_cases.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            test_cases.delete,
        )
        self.bulk_delete = async_to_streamed_response_wrapper(
            test_cases.bulk_delete,
        )
        self.bulk_update = async_to_streamed_response_wrapper(
            test_cases.bulk_update,
        )
        self.bulk_move = async_to_streamed_response_wrapper(
            test_cases.bulk_move,
        )

    @cached_property
    def comments(self) -> AsyncCommentsResourceWithStreamingResponse:
        return AsyncCommentsResourceWithStreamingResponse(self._test_cases.comments)



from __future__ import annotations

from typing import List, Union, Literal, Optional
from datetime import datetime

import httpx

from ..types import (
    Agent,
    Dataset,
    Evaluation,
    APIResponse,
    FrequencyOption,
    ScheduledEvaluation,
    APIResponseWithIncluded,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from ._included import embed_included_list, embed_included_single
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.scheduled_evaluation import (
    LastExecutionStatusParam,
    ScheduledEvaluationListParams,
    ScheduledEvaluationCreateParams,
    ScheduledEvaluationUpdateParams,
    ScheduledEvaluationRetrieveParams,
    ScheduledEvaluationBulkDeleteParams,
    ScheduledEvaluationListEvaluationsParams,
)

__all__ = ["ScheduledEvaluationsResource", "AsyncScheduledEvaluationsResource"]


class ScheduledEvaluationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ScheduledEvaluationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return ScheduledEvaluationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ScheduledEvaluationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return ScheduledEvaluationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        project_id: str,
        name: str,
        agent_id: str,
        dataset_id: str,
        tags: SequenceNotStr[str] | Omit = omit,
        run_count: int | Omit = omit,
        frequency: FrequencyOption,
        time: str,
        day_of_month: Optional[int] | Omit = omit,
        day_of_week: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduledEvaluation:
        """Create a new scheduled evaluation for recurring agent testing.

        Parameters
        ----------
        project_id : str
            Project ID to use for the scheduled evaluation.
        name : str
            Name of the scheduled evaluation.
        agent_id : str
            Agent ID to use for the scheduled evaluation.
        dataset_id : str
            Dataset ID to use for the scheduled evaluation.
        tags : SequenceNotStr[str] | Omit
            List of tags to apply to the scheduled evaluation.
        run_count : int | Omit
            The number of times to run each test case. This is useful to get a more
            accurate result when the chatbot's generation is not deterministic. Testing
            stops at the first failure. If all runs pass, the test case is considered
            successful.
        frequency : FrequencyOption
            Frequency of the scheduled evaluation.
        time : str
            Time of the scheduled evaluation.
        day_of_month : int | None | Omit
            Day of the month to run the scheduled evaluation.
        day_of_week : int | None | Omit
            Day of the week to run the scheduled evaluation.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScheduledEvaluation
            The newly created scheduled evaluation.
        """
        response = self._post(
            "/v2/scheduled-evaluations",
            body=maybe_transform(
                {
                    "agent_id": agent_id,
                    "dataset_id": dataset_id,
                    "frequency": frequency,
                    "name": name,
                    "project_id": project_id,
                    "time": time,
                    "day_of_month": day_of_month,
                    "day_of_week": day_of_week,
                    "run_count": run_count,
                    "tags": tags,
                },
                ScheduledEvaluationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[ScheduledEvaluation],
        )

        return self._unwrap(response)

    def retrieve(
        self,
        scheduled_evaluation_id: str,
        include: Optional[List[Literal["evaluations"]]] | Omit = omit,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduledEvaluation:
        """Retrieve a scheduled evaluation by its ID.

        Parameters
        ----------
        scheduled_evaluation_id : str
            ID of the scheduled evaluation to retrieve.
        include : list[Literal["evaluations"]] | None | Omit
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScheduledEvaluation
            The requested scheduled evaluation.

        Raises
        ------
        ValueError
            If ``scheduled_evaluation_id`` is empty.
        """
        if not scheduled_evaluation_id:
            raise ValueError(
                f"Expected a non-empty value for `scheduled_evaluation_id` but received {scheduled_evaluation_id!r}"
            )
        response = self._get(
            f"/v2/scheduled-evaluations/{scheduled_evaluation_id}",
            options=make_request_options(
                query=maybe_transform({"include": include}, ScheduledEvaluationRetrieveParams),
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=APIResponseWithIncluded[ScheduledEvaluation, List[APIResponse[Evaluation]]],
        )

        if include is not omit and include:
            response = embed_included_single(response, id_getter=lambda scheduled: scheduled.id)

        return self._unwrap(response)

    def update(
        self,
        scheduled_evaluation_id: str,
        *,
        name: Optional[str] | Omit = omit,
        run_count: Optional[int] | Omit = omit,
        frequency: Optional[FrequencyOption] | Omit = omit,
        time: Optional[str] | Omit = omit,
        day_of_week: Optional[int] | Omit = omit,
        day_of_month: Optional[int] | Omit = omit,
        last_execution_at: Union[str, datetime, None] | Omit = omit,
        last_execution_status: Optional[LastExecutionStatusParam] | Omit = omit,
        paused: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduledEvaluation:
        """Update an existing scheduled evaluation's configuration.

        Parameters
        ----------
        scheduled_evaluation_id : str
            ID of the scheduled evaluation to update.
        name : str | None | Omit
            Name of the scheduled evaluation.
        run_count : int | None | Omit
            The number of times to run each test case. This is useful to get a more
            accurate result when the chatbot's generation is not deterministic. Testing
            stops at the first failure. If all runs pass, the test case is considered
            successful.
        frequency : FrequencyOption | None | Omit
            Frequency of the scheduled evaluation.
        time : str | None | Omit
            Time of the scheduled evaluation.
        day_of_week : int | None | Omit
            Day of the week to run the scheduled evaluation.
        day_of_month : int | None | Omit
            Day of the month to run the scheduled evaluation.
        last_execution_at : str | datetime | None | Omit
            The date and time of the last execution of the scheduled evaluation.
        last_execution_status : LastExecutionStatusParam | None | Omit
            The status of the last execution of the scheduled evaluation.
        paused : bool | None | Omit
            Whether the scheduled evaluation is paused.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScheduledEvaluation
            The updated scheduled evaluation.

        Raises
        ------
        ValueError
            If ``scheduled_evaluation_id`` is empty.
        """
        if not scheduled_evaluation_id:
            raise ValueError(
                f"Expected a non-empty value for `scheduled_evaluation_id` but received {scheduled_evaluation_id!r}"
            )
        response = self._patch(
            f"/v2/scheduled-evaluations/{scheduled_evaluation_id}",
            body=maybe_transform(
                {
                    "day_of_month": day_of_month,
                    "day_of_week": day_of_week,
                    "frequency": frequency,
                    "last_execution_at": last_execution_at,
                    "last_execution_status": last_execution_status,
                    "name": name,
                    "paused": paused,
                    "run_count": run_count,
                    "time": time,
                },
                ScheduledEvaluationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[ScheduledEvaluation],
        )

        return self._unwrap(response)

    def list(
        self,
        *,
        project_id: str,
        include: Optional[List[Literal["evaluations"]]] | Omit = omit,
        last_days: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[ScheduledEvaluation]:
        """List all scheduled evaluations for a project.

        Parameters
        ----------
        project_id : str
            Project ID to use for the scheduled evaluation.
        include : list[Literal["evaluations"]] | None | Omit
            Related resources to include in response.
        last_days : int | None | Omit
            Number of days to include in the response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[ScheduledEvaluation]
            A list of scheduled evaluations for the project.
        """
        response = self._get(
            "/v2/scheduled-evaluations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"project_id": project_id, "include": include, "last_days": last_days},
                    ScheduledEvaluationListParams,
                ),
            ),
            cast_to=APIResponseWithIncluded[List[ScheduledEvaluation], List[APIResponse[Evaluation]]],
        )

        if include is not omit and include:
            response = embed_included_list(response, id_getter=lambda scheduled: scheduled.id)

        return self._unwrap(response)

    def delete(
        self,
        scheduled_evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a scheduled evaluation by its ID.

        Parameters
        ----------
        scheduled_evaluation_id : str
            ID of the scheduled evaluation to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``scheduled_evaluation_id`` is empty.
        """
        if not scheduled_evaluation_id:
            raise ValueError(
                f"Expected a non-empty value for `scheduled_evaluation_id` but received {scheduled_evaluation_id!r}"
            )
        response = self._delete(
            f"/v2/scheduled-evaluations/{scheduled_evaluation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def bulk_delete(
        self,
        *,
        scheduled_evaluation_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple scheduled evaluations at once.

        Parameters
        ----------
        scheduled_evaluation_ids : SequenceNotStr[str]
            IDs of the scheduled evaluations to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = self._delete(
            "/v2/scheduled-evaluations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"scheduled_evaluation_ids": scheduled_evaluation_ids},
                    ScheduledEvaluationBulkDeleteParams,
                ),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def list_evaluations(
        self,
        scheduled_evaluation_id: str,
        include: Optional[List[Literal["agent", "dataset"]]] | Omit = omit,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Evaluation]:
        """List all evaluation runs triggered by a specific scheduled evaluation.

        Parameters
        ----------
        scheduled_evaluation_id : str
            ID of the scheduled evaluation to list evaluations for.
        include : list[Literal["agent", "dataset"]] | None | Omit
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[Evaluation]
            A list of evaluations triggered by the scheduled evaluation.

        Raises
        ------
        ValueError
            If ``scheduled_evaluation_id`` is empty.
        """
        if not scheduled_evaluation_id:
            raise ValueError(
                f"Expected a non-empty value for `scheduled_evaluation_id` but received {scheduled_evaluation_id!r}"
            )

        response = self._get(
            f"/v2/scheduled-evaluations/{scheduled_evaluation_id}/evaluations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"include": include},
                    ScheduledEvaluationListEvaluationsParams,
                ),
            ),
            cast_to=APIResponseWithIncluded[List[Evaluation], APIResponse[Agent | Dataset]],
        )

        if include is not omit and include:
            response = embed_included_list(response, id_getter=lambda evaluation: evaluation.id)

        return self._unwrap(response)


class AsyncScheduledEvaluationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncScheduledEvaluationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncScheduledEvaluationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncScheduledEvaluationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncScheduledEvaluationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        project_id: str,
        name: str,
        agent_id: str,
        dataset_id: str,
        tags: SequenceNotStr[str] | Omit = omit,
        run_count: int | Omit = omit,
        frequency: FrequencyOption,
        time: str,
        day_of_month: Optional[int] | Omit = omit,
        day_of_week: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduledEvaluation:
        """Create a new scheduled evaluation for recurring agent testing.

        Parameters
        ----------
        project_id : str
            Project ID to use for the scheduled evaluation.
        name : str
            Name of the scheduled evaluation.
        agent_id : str
            Agent ID to use for the scheduled evaluation.
        dataset_id : str
            Dataset ID to use for the scheduled evaluation.
        tags : SequenceNotStr[str] | Omit
            List of tags to apply to the scheduled evaluation.
        run_count : int | Omit
            The number of times to run each test case. This is useful to get a more
            accurate result when the chatbot's generation is not deterministic. Testing
            stops at the first failure. If all runs pass, the test case is considered
            successful.
        frequency : FrequencyOption
            Frequency of the scheduled evaluation.
        time : str
            Time of the scheduled evaluation.
        day_of_month : int | None | Omit
            Day of the month to run the scheduled evaluation.
        day_of_week : int | None | Omit
            Day of the week to run the scheduled evaluation.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScheduledEvaluation
            The newly created scheduled evaluation.
        """
        response = await self._post(
            "/v2/scheduled-evaluations",
            body=await async_maybe_transform(
                {
                    "agent_id": agent_id,
                    "dataset_id": dataset_id,
                    "frequency": frequency,
                    "name": name,
                    "project_id": project_id,
                    "time": time,
                    "day_of_month": day_of_month,
                    "day_of_week": day_of_week,
                    "run_count": run_count,
                    "tags": tags,
                },
                ScheduledEvaluationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[ScheduledEvaluation],
        )

        return self._unwrap(response)

    async def retrieve(
        self,
        scheduled_evaluation_id: str,
        include: Optional[List[Literal["evaluations"]]] | Omit = omit,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduledEvaluation:
        """Retrieve a scheduled evaluation by its ID.

        Parameters
        ----------
        scheduled_evaluation_id : str
            ID of the scheduled evaluation to retrieve.
        include : list[Literal["evaluations"]] | None | Omit
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScheduledEvaluation
            The requested scheduled evaluation.

        Raises
        ------
        ValueError
            If ``scheduled_evaluation_id`` is empty.
        """
        if not scheduled_evaluation_id:
            raise ValueError(
                f"Expected a non-empty value for `scheduled_evaluation_id` but received {scheduled_evaluation_id!r}"
            )
        response = await self._get(
            f"/v2/scheduled-evaluations/{scheduled_evaluation_id}",
            options=make_request_options(
                query=maybe_transform({"include": include}, ScheduledEvaluationRetrieveParams),
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=APIResponseWithIncluded[ScheduledEvaluation, List[APIResponse[Evaluation]]],
        )

        if include is not omit and include:
            response = embed_included_single(response, id_getter=lambda scheduled: scheduled.id)

        return self._unwrap(response)

    async def update(
        self,
        scheduled_evaluation_id: str,
        *,
        name: Optional[str] | Omit = omit,
        run_count: Optional[int] | Omit = omit,
        frequency: Optional[FrequencyOption] | Omit = omit,
        time: Optional[str] | Omit = omit,
        day_of_week: Optional[int] | Omit = omit,
        day_of_month: Optional[int] | Omit = omit,
        last_execution_at: Union[str, datetime, None] | Omit = omit,
        last_execution_status: Optional[LastExecutionStatusParam] | Omit = omit,
        paused: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduledEvaluation:
        """Update an existing scheduled evaluation's configuration.

        Parameters
        ----------
        scheduled_evaluation_id : str
            ID of the scheduled evaluation to update.
        name : str | None | Omit
            Name of the scheduled evaluation.
        run_count : int | None | Omit
            The number of times to run each test case. This is useful to get a more
            accurate result when the chatbot's generation is not deterministic. Testing
            stops at the first failure. If all runs pass, the test case is considered
            successful.
        frequency : FrequencyOption | None | Omit
            Frequency of the scheduled evaluation.
        time : str | None | Omit
            Time of the scheduled evaluation.
        day_of_week : int | None | Omit
            Day of the week to run the scheduled evaluation.
        day_of_month : int | None | Omit
            Day of the month to run the scheduled evaluation.
        last_execution_at : str | datetime | None | Omit
            The date and time of the last execution of the scheduled evaluation.
        last_execution_status : LastExecutionStatusParam | None | Omit
            The status of the last execution of the scheduled evaluation.
        paused : bool | None | Omit
            Whether the scheduled evaluation is paused.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScheduledEvaluation
            The updated scheduled evaluation.

        Raises
        ------
        ValueError
            If ``scheduled_evaluation_id`` is empty.
        """
        if not scheduled_evaluation_id:
            raise ValueError(
                f"Expected a non-empty value for `scheduled_evaluation_id` but received {scheduled_evaluation_id!r}"
            )
        response = await self._patch(
            f"/v2/scheduled-evaluations/{scheduled_evaluation_id}",
            body=await async_maybe_transform(
                {
                    "day_of_month": day_of_month,
                    "day_of_week": day_of_week,
                    "frequency": frequency,
                    "last_execution_at": last_execution_at,
                    "last_execution_status": last_execution_status,
                    "name": name,
                    "paused": paused,
                    "run_count": run_count,
                    "time": time,
                },
                ScheduledEvaluationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[ScheduledEvaluation],
        )

        return self._unwrap(response)

    async def list(
        self,
        *,
        project_id: str,
        include: Optional[List[Literal["evaluations"]]] | Omit = omit,
        last_days: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[ScheduledEvaluation]:
        """List all scheduled evaluations for a project.

        Parameters
        ----------
        project_id : str
            Project ID to use for the scheduled evaluation.
        include : list[Literal["evaluations"]] | None | Omit
            Related resources to include in response.
        last_days : int | None | Omit
            Number of days to include in the response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[ScheduledEvaluation]
            A list of scheduled evaluations for the project.
        """
        response = await self._get(
            "/v2/scheduled-evaluations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"project_id": project_id, "include": include, "last_days": last_days},
                    ScheduledEvaluationListParams,
                ),
            ),
            cast_to=APIResponseWithIncluded[List[ScheduledEvaluation], List[APIResponse[Evaluation]]],
        )

        if include is not omit and include:
            response = embed_included_list(response, id_getter=lambda scheduled: scheduled.id)

        return self._unwrap(response)

    async def delete(
        self,
        scheduled_evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a scheduled evaluation by its ID.

        Parameters
        ----------
        scheduled_evaluation_id : str
            ID of the scheduled evaluation to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``scheduled_evaluation_id`` is empty.
        """
        if not scheduled_evaluation_id:
            raise ValueError(
                f"Expected a non-empty value for `scheduled_evaluation_id` but received {scheduled_evaluation_id!r}"
            )
        response = await self._delete(
            f"/v2/scheduled-evaluations/{scheduled_evaluation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def bulk_delete(
        self,
        *,
        scheduled_evaluation_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple scheduled evaluations at once.

        Parameters
        ----------
        scheduled_evaluation_ids : SequenceNotStr[str]
            IDs of the scheduled evaluations to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = await self._delete(
            "/v2/scheduled-evaluations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"scheduled_evaluation_ids": scheduled_evaluation_ids},
                    ScheduledEvaluationBulkDeleteParams,
                ),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def list_evaluations(
        self,
        scheduled_evaluation_id: str,
        include: Optional[List[Literal["agent", "dataset"]]] | Omit = omit,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Evaluation]:
        """List all evaluation runs triggered by a specific scheduled evaluation.

        Parameters
        ----------
        scheduled_evaluation_id : str
            ID of the scheduled evaluation to list evaluations for.
        include : list[Literal["agent", "dataset"]] | None | Omit
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[Evaluation]
            A list of evaluations triggered by the scheduled evaluation.

        Raises
        ------
        ValueError
            If ``scheduled_evaluation_id`` is empty.
        """
        if not scheduled_evaluation_id:
            raise ValueError(
                f"Expected a non-empty value for `scheduled_evaluation_id` but received {scheduled_evaluation_id!r}"
            )

        response = await self._get(
            f"/v2/scheduled-evaluations/{scheduled_evaluation_id}/evaluations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include": include},
                    ScheduledEvaluationListEvaluationsParams,
                ),
            ),
            cast_to=APIResponseWithIncluded[List[Evaluation], APIResponse[Agent | Dataset]],
        )

        if include is not omit and include:
            response = embed_included_list(response, id_getter=lambda evaluation: evaluation.id)

        return self._unwrap(response)


class ScheduledEvaluationsResourceWithRawResponse:
    def __init__(self, scheduled_evaluations: ScheduledEvaluationsResource) -> None:
        self._scheduled_evaluations = scheduled_evaluations

        self.create = to_raw_response_wrapper(
            scheduled_evaluations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            scheduled_evaluations.retrieve,
        )
        self.update = to_raw_response_wrapper(
            scheduled_evaluations.update,
        )
        self.list = to_raw_response_wrapper(
            scheduled_evaluations.list,
        )
        self.delete = to_raw_response_wrapper(
            scheduled_evaluations.delete,
        )
        self.bulk_delete = to_raw_response_wrapper(
            scheduled_evaluations.bulk_delete,
        )
        self.list_evaluations = to_raw_response_wrapper(
            scheduled_evaluations.list_evaluations,
        )


class AsyncScheduledEvaluationsResourceWithRawResponse:
    def __init__(self, scheduled_evaluations: AsyncScheduledEvaluationsResource) -> None:
        self._scheduled_evaluations = scheduled_evaluations

        self.create = async_to_raw_response_wrapper(
            scheduled_evaluations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            scheduled_evaluations.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            scheduled_evaluations.update,
        )
        self.list = async_to_raw_response_wrapper(
            scheduled_evaluations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            scheduled_evaluations.delete,
        )
        self.bulk_delete = async_to_raw_response_wrapper(
            scheduled_evaluations.bulk_delete,
        )
        self.list_evaluations = async_to_raw_response_wrapper(
            scheduled_evaluations.list_evaluations,
        )


class ScheduledEvaluationsResourceWithStreamingResponse:
    def __init__(self, scheduled_evaluations: ScheduledEvaluationsResource) -> None:
        self._scheduled_evaluations = scheduled_evaluations

        self.create = to_streamed_response_wrapper(
            scheduled_evaluations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            scheduled_evaluations.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            scheduled_evaluations.update,
        )
        self.list = to_streamed_response_wrapper(
            scheduled_evaluations.list,
        )
        self.delete = to_streamed_response_wrapper(
            scheduled_evaluations.delete,
        )
        self.bulk_delete = to_streamed_response_wrapper(
            scheduled_evaluations.bulk_delete,
        )
        self.list_evaluations = to_streamed_response_wrapper(
            scheduled_evaluations.list_evaluations,
        )


class AsyncScheduledEvaluationsResourceWithStreamingResponse:
    def __init__(self, scheduled_evaluations: AsyncScheduledEvaluationsResource) -> None:
        self._scheduled_evaluations = scheduled_evaluations

        self.create = async_to_streamed_response_wrapper(
            scheduled_evaluations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            scheduled_evaluations.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            scheduled_evaluations.update,
        )
        self.list = async_to_streamed_response_wrapper(
            scheduled_evaluations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            scheduled_evaluations.delete,
        )
        self.bulk_delete = async_to_streamed_response_wrapper(
            scheduled_evaluations.bulk_delete,
        )
        self.list_evaluations = async_to_streamed_response_wrapper(
            scheduled_evaluations.list_evaluations,
        )



"""Check, assertion, and annotation types."""

from typing import Any, Dict, List, Union, Literal, Iterable, Optional, TypeAlias, TypedDict, cast
from datetime import datetime
from typing_extensions import Required

import pydantic

from .common import TaskState
from .._types import SequenceNotStr
from .._models import BaseModel

__all__ = [
    "Check",
    "CheckResult",
    "CheckConfig",
    "CheckConfigParam",
    "CheckType",
    "CheckTypeParam",
    "ConformityParams",
    "ConformityParamsParam",
    "CorrectnessParams",
    "CorrectnessParamsParam",
    "GroundednessParams",
    "GroundednessParamsParam",
    "StringMatchParams",
    "StringMatchParamsParam",
    "SemanticSimilarityParams",
    "SemanticSimilarityParamsParam",
    "MetadataParams",
    "MetadataParamsParam",
    "JsonPathRule",
    "JsonPathRuleParam",
    "OutputAnnotation",
    "TestCaseCheckConfig",
    "TestCaseCheckConfigParam",
    "CheckListParams",
    "CheckCreateParams",
    "CheckUpdateParams",
    "CheckBulkDeleteParams",
]


# ---------------------------------------------------------------------------
# Assertion parameter models (BaseModel – used in responses)
# ---------------------------------------------------------------------------


class ConformityParams(BaseModel):
    rules: List[str]
    type: Optional[Literal["conformity"]] = None


class CorrectnessParams(BaseModel):
    reference: str
    type: Optional[Literal["correctness"]] = None


class GroundednessParams(BaseModel):
    context: str
    type: Optional[Literal["groundedness"]] = None


class StringMatchParams(BaseModel):
    keyword: str
    type: Optional[Literal["string_match"]] = None


class SemanticSimilarityParams(BaseModel):
    reference: str
    threshold: Optional[float] = None
    type: Optional[Literal["semantic_similarity"]] = None


class JsonPathRule(BaseModel):
    expected_value: Union[bool, float, str]
    expected_value_type: Literal["string", "number", "boolean"]
    json_path: str


class MetadataParams(BaseModel):
    json_path_rules: List[JsonPathRule]
    type: Optional[Literal["metadata"]] = None


CheckType: TypeAlias = Union[
    CorrectnessParams,
    ConformityParams,
    GroundednessParams,
    StringMatchParams,
    MetadataParams,
    SemanticSimilarityParams,
]


# ---------------------------------------------------------------------------
# Assertion parameter TypedDicts (used in requests)
# ---------------------------------------------------------------------------


class ConformityParamsParam(TypedDict, total=False):
    rules: Required[SequenceNotStr[str]]
    type: Literal["conformity"]


class CorrectnessParamsParam(TypedDict, total=False):
    reference: Required[str]
    type: Literal["correctness"]


class GroundednessParamsParam(TypedDict, total=False):
    context: Required[str]
    type: Literal["groundedness"]


class StringMatchParamsParam(TypedDict, total=False):
    keyword: Required[str]
    type: Literal["string_match"]


class SemanticSimilarityParamsParam(TypedDict, total=False):
    reference: Required[str]
    threshold: float
    type: Literal["semantic_similarity"]


class JsonPathRuleParam(TypedDict, total=False):
    expected_value: Required[Union[bool, float, str]]
    expected_value_type: Required[Literal["string", "number", "boolean"]]
    json_path: Required[str]


class MetadataParamsParam(TypedDict, total=False):
    json_path_rules: Required[Iterable[JsonPathRuleParam]]
    type: Literal["metadata"]


CheckTypeParam: TypeAlias = Union[
    CorrectnessParamsParam,
    ConformityParamsParam,
    GroundednessParamsParam,
    StringMatchParamsParam,
    MetadataParamsParam,
    SemanticSimilarityParamsParam,
]


# ---------------------------------------------------------------------------
# Output annotation
# ---------------------------------------------------------------------------


class OutputAnnotation(BaseModel):
    end_char_index: int
    label: str
    start_char_index: int
    text: str
    type: Literal["output", "context"]


# ---------------------------------------------------------------------------
# Check models
# ---------------------------------------------------------------------------


class Check(BaseModel):
    id: str
    built_in: bool
    created_at: datetime
    description: Optional[str] = None
    identifier: str
    name: str
    params: Dict[str, Any] = {}
    project_id: str
    updated_at: datetime

    @pydantic.model_validator(mode="before")
    @classmethod
    def _convert_assertions(cls, data: Any) -> Any:  # noqa: ANN401
        if not isinstance(data, dict):
            return data
        d = cast(Dict[str, Any], data)
        if "params" in d:
            return d
        assertions: list[Any] = d.get("assertions") or []
        return {**d, "params": assertions[0] if assertions else {}}


class CheckResult(BaseModel):
    name: str
    display_name: Optional[str] = None
    status: TaskState
    passed: Optional[bool] = None
    error: Optional[str] = None
    reason: Optional[str] = None
    annotations: Optional[List[OutputAnnotation]] = None


# ---------------------------------------------------------------------------
# Test case check config
# ---------------------------------------------------------------------------


class TestCaseCheckConfig(BaseModel):
    __test__ = False
    identifier: str
    assertions: Optional[List[CheckType]] = None
    enabled: Optional[bool] = None


class TestCaseCheckConfigParam(TypedDict, total=False):
    identifier: Required[str]
    assertions: Optional[Iterable[CheckTypeParam]]
    enabled: bool


# ---------------------------------------------------------------------------
# User-facing check config (simplified format with params instead of assertions)
# ---------------------------------------------------------------------------


def _extract_check_params(check: Dict[str, Any]) -> Dict[str, Any]:
    """Extract params from the first assertion, stripping the ``type`` key."""
    assertions: list[Any] = check.get("assertions") or []
    if not assertions:
        return {}
    first: Any = assertions[0]
    if isinstance(first, BaseModel):
        return first.model_dump(exclude={"type"}, exclude_none=True)
    if isinstance(first, dict):
        return {k: v for k, v in cast(Dict[str, Any], first).items() if k != "type"}
    return {}


class CheckConfig(BaseModel):
    """Simplified check configuration with ``params`` instead of raw ``assertions``."""

    identifier: str
    enabled: Optional[bool] = None
    params: Dict[str, Any] = {}

    @pydantic.model_validator(mode="before")
    @classmethod
    def _convert_assertions(cls, data: Any) -> Any:  # noqa: ANN401
        if not isinstance(data, dict):
            return data
        d = cast(Dict[str, Any], data)
        if "params" in d:
            return d
        return {**d, "params": _extract_check_params(d)}


class CheckConfigParam(TypedDict, total=False):
    identifier: Required[str]
    enabled: bool
    params: Dict[str, Any]


def _check_params_to_api(  # pyright: ignore[reportUnusedFunction]
    checks: Iterable[CheckConfigParam],
) -> list[Dict[str, Any]]:
    """Convert user-facing CheckParam dicts to the API assertions format."""
    return [
        {
            "identifier": check["identifier"],
            "enabled": check.get("enabled", True),
            **(
                {"assertions": [{"type": check["identifier"], **check.get("params", {})}]}
                if check.get("params")
                else {}
            ),
        }
        for check in checks
    ]


# ---------------------------------------------------------------------------
# Check params
# ---------------------------------------------------------------------------


class CheckListParams(TypedDict, total=False):
    project_id: Required[str]
    filter_builtin: bool


class CheckCreateParams(TypedDict, total=False):
    assertions: Required[Iterable[CheckTypeParam]]
    description: Optional[str]
    identifier: Required[str]
    name: Required[str]
    project_id: Required[str]


class CheckUpdateParams(TypedDict, total=False):
    assertions: Optional[Iterable[CheckTypeParam]]
    description: Optional[str]
    identifier: Optional[str]
    name: Optional[str]


class CheckBulkDeleteParams(TypedDict, total=False):
    check_ids: Required[SequenceNotStr[str]]



from __future__ import annotations

from urllib.parse import unquote

from giskard_hub import HubClient, AsyncHubClient


def test_hubclient_uses_repeat_array_query_params() -> None:
    client = HubClient(base_url="http://example.com", api_key="test")
    try:
        assert unquote(client.qs.stringify({"include": ["agent", "dataset"]})) == "include=agent&include=dataset"
        assert unquote(client.qs.stringify({"include": ["dataset", "agent"]})) == "include=dataset&include=agent"
    finally:
        client.close()


async def test_asynchubclient_uses_repeat_array_query_params() -> None:
    client = AsyncHubClient(base_url="http://example.com", api_key="test")
    try:
        assert unquote(client.qs.stringify({"include": ["agent", "dataset"]})) == "include=agent&include=dataset"
        assert unquote(client.qs.stringify({"include": ["dataset", "agent"]})) == "include=dataset&include=agent"
    finally:
        await client.close()
